from math import *

class Rationnel:
    def __init__(self,a,b=1):
        """constructeur - code appelé lors de la construction d'un nouveau rationnel"""
        self.numerateur=a
        if b==0:
            raise ValueError('Un dénominateur ne peut pas être nul')
        self.denominateur=b

    def affiche(self,nbchiffres):
        """affichage tronqué à nbchiffres après la virgule"""
        puiss=10**nbchiffres
        t=self.numerateur*puiss//self.denominateur
        (ent,dec)=(t//puiss,t%puiss)
        print(str(ent),'.',str(dec),sep='')

    def simplifie(self):
        """met le nombre rationnel sous forme irréductible"""
        g=gcd(self.numerateur,self.denominateur)
        self.numerateur//=g
        self.denominateur//=g
        if self.denominateur<0:
            self.denominateur*=-1
            self.numerateur*=-1

    def estentier(self):
        """teste si le rationnel est entier"""
        self.simplifie()
        return self.denominateur==1

    def estnul(self):
        """teste si le rationnel est nul"""
        return self.numerateur==0

    def __str__(self):
        """conversion en chaine de caractères (utilisée par print)"""
        self.simplifie()
        return str(self.numerateur)+'/'+str(self.denominateur)

    def __neg__(self):
        """soustraction"""
        return Rationnel(-self.numerateur,self.denominateur)
        
    def __truediv__(self,other):
        """division avec le signe /"""
        return Rationnel(self.numerateur*other.denominateur,self.denominateur*other.numerateur)

    def __eq__(self,other):
        """test d'égalité"""
        return (self.numerateur*other.denominateur==other.numerateur*self.denominateur)

    def __add__(self,other):
        """addition"""
        return Rationnel(self.numerateur*other.denominateur+other.numerateur*self.denominateur,self.denominateur*other.denominateur)

    def __mul__(self,other):
        """produit"""
        return Rationnel(self.numerateur*other.numerateur,self.denominateur*other.denominateur)
        
    
######

def heron(a,n):
    """calcule le n eme terme de la suite de Héron, où u0=a est un entier naturel non-nul"""
    u=Rationnel(1)
    for _ in range(n):
        u=Rationnel(1,2)*(u+Rationnel(a)/u)
    return u

r=heron(2,10)
r.affiche(400)


